import sys, os, glob




from Keywords import Keyword as Kw, Set as KwS
from Prophet.Categories import *
from Prophet import msg




import Prophet




class _App(Prophet.Application) :
	"""A simple application that may have different executables"""

	def __init__(self) :
		self.keywords |= KwS(Legacy)

		try :
			exes = self.exes
		except AttributeError :
			exes = [self.__class__.__name__]

		for x in exes :
			candidates = []
			for p in Prophet.prefixes :
				for b in Prophet.prefixes.binDirs :
					d = os.path.join(p, b)
					for g in glob.glob(os.path.join(d, x)) :
						t = (p, d, g)
						if self.isValid(*t) :
							candidates.append(t)
			try :
				pfx, dir, exe = self.mostSuitable(candidates)

				path = os.path.join(dir, exe)
				self.exename = path
				if Prophet.omitExecPath :
					try :
						# Make sure the chosen candidate is actually the one that
						# will be picked up via PATH envar
						if os.path.samefile(path, Prophet.locateExec(exe)) :
							self.exename = exe
					except (OSError, Prophet.NotFound) :
						pass

				try :
					self.name
				except AttributeError :
					self.name = self.__class__.__name__.capitalize()

				super(_App, self).__init__()

				return
			except Prophet.NotFound :
				pass
		raise Prophet.NotFound("No suitable executable found")
		
	
	def isValid(self, pfx, dir, exe) :
		"""Return True when the executable is considered valid (existent, callable etc.) and False otherwise
			pfx -- a prefix where the executable is (supposedly) found, ex. /usr/local
			dir -- a directory containing the file, ex. /usr/local/bin
			exe -- a file w/o path, ex. gqview"""
		try :
			return Prophet.isExec(os.path.join(dir, exe))
		except Prophet.NotFound :
			return False

	def mostSuitable(self, candidates) :
		"""Given the list of (pfx, dir, exe), choose and return a tuple of the most suitable candidate"""
		# Assume that candidates come in order of descending importance, hence the first one wins
		if len(candidates) :
			return candidates[0]
		else :
			raise Prophet.NotFound("Empty list of candidates passed")




class _VersApp(_App) :
	"""An application that usually (but not always) has versioned executable, ex. tclsh8.4"""




class _ConsoleApp(Prophet.Application) :
	"""Mixin class for the console application that must be run in terminal"""
	def __init__(self) :
		try :
			self.terminal
		except AttributeError :
			self.terminal = True
		self.keywords |= KwS(ConsoleOnly)
		super(_ConsoleApp, self).__init__()




class _X11App(Prophet.Application) :
	"""Mixin class for the X11 GUI application"""
	def __init__(self) :
		try :
			self.terminal
		except AttributeError :
			self.terminal = False
		super(_X11App, self).__init__()




__all__ = ["Development", "Editor", "Emulator", "Multimedia", "Shell"]




entries = [] # List of all legacy entries found




def _register(module) :
	"""Import and store the specified module along with all its submodules"""

	try :
		names = module.__all__
	except AttributeError :
		names = []

	for k, v in module.__dict__.items() :
		if not k.startswith("_") and type(v) == type and issubclass(v, Prophet.Application) :
			entries.append(v)

	for x in names :
		name = module.__name__ + "." + x
		try :
			__import__(name)
		except ImportError :
			raise ImportError("No module named " + name)
		imp = sys.modules[name]
		_register(imp)
		




def setup() :
	_register(sys.modules[__name__])




def scan() :
	result = []
	msg("  legacy...", newline = False)
	for x in entries :
		try :
			result.append( x() )
		except Prophet.NotFound :
			pass
	msg(" %d apps found" % len(result))
	return result
